#!/bin/bash
function call() {
  echo -e "$(date '+%d.%m.%Y %H:%M:%S') [RP Maint] $@"
}
tstamp="$(/bin/date '+%d-%m-%Y_%H-%M-%S')"
declare -a domains
declare -a server_names
declare -A ip_map
declare -a mail_domains
ipmatch="^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$"

if [ $# -eq 0 ]
  then
    call "No arguments supplied. Exiting here"
    exit 1
fi


ssl_mode=""
srv_id=$(cat /usr/local/ispconfig/server/lib/config.inc.php | grep server_id | grep -Eo '[[:digit:]]+')
if [[ -z ${srv_id} ]]; then
  call "Not able to tell the server_id. Exiting here!"
  exit 1
fi

while [[ $# -ge 1 ]]; do
  INPUT="$1"
  case $INPUT in
    --proxy)
      proxy="1"
      proxy_ip="${2}"
      mode="ReverseProxy"
      shift
      shift
      ;;
    --maint)
      maint="1"
      mode="Maintenance Mode"
      shift
      shift
      ;;
    --ispcopy_ini)
      mode="ISPCopyProxy"
      shift
      shift
      ;;
    --routing)
      mode="EnableRouting"
      shift
      shift
      ;;
    --clearcrons)
      mode="ClearCrons"
      shift
      shift
      ;;
    *)
    call "Wrong input: $INPUT"
    call "Please use either --proxy ip.ip.ip.ip or --maint only!"
    exit 1
    ;;
  esac
done

if [[ ! -z ${proxy_ip} && ! -z ${maint} ]]; then
  call "Cant use --proxy and --maint at the same time. Please use either --proxy ip.ip.ip.ip or --maint only!"
  exit 1
fi

if [[ ${proxy} == "1" ]]; then
  r_ip=$(echo "${proxy_ip}" | sed -E -e 's/[[:blank:]]+//g'| grep -Eo "${ipmatch}")
  if [[ ${proxy_ip} == ${r_ip} ]]; then
     reverse_ip=${r_ip}
  else
    call "Wrong input: ${proxy_ip}"
    call "Please use either --proxy ip.ip.ip.ip or --maint only!"
    exit 1
  fi
fi


call "${mode} requested"
if [[ ${mode} == "EnableRouting" ]]; then
  ispc_ini_file="/root/ispcopy/ispcopy_input.ini"
  if [[ ! -f ${ispc_ini_file} ]]; then
    call "${mode} requires ${ispc_ini_file} to be available. Make sure the file is in place."
    call "Output of ls: \n-----\n$(ls -lah "/root/ispcopy/")\n-----"
    exit 1
  else
    routing_target=$(awk -F "=" '/target_server_hostname/{print$2}' ${ispc_ini_file})
    if [[ -z ${routing_target} ]]; then
      call "Failed to parse routing target in ${ispc_ini_file}. Exiting here!"
      exit 1
    else
      call "Using ISPCopy .ini file to enable mail routing to target: ${routing_target}."
    fi
  fi

  for i in  $(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select domain from dbispconfig.mail_domain WHERE server_id = ${srv_id};"); do
    mail_domains+=("$i")
  done
  call "Routing enabled for:"
  for d in ${mail_domains[@]}; do
    mysql --defaults-extra-file=/etc/mysql/debian.cnf -e "INSERT INTO dbispconfig.mail_transport (sys_userid, sys_groupid, sys_perm_user, sys_perm_group, server_id, domain, transport, sort_order, active) VALUES (1, 1, 'riud', 'riud', ${srv_id},'${d}', 'smtp:[${routing_target}]', 5, 'y');"
    call "✔ ${d} "
  done
  exit 0
fi

if [[ ${mode} == "ClearCrons" ]]; then
  cron_ba="/root/cron_backup_${tstamp}"
  if [[ $(find /etc/cron.d/ -iname "ispc*" | wc -l) -gt 0 || $(find /var/spool/cron/crontabs/ -regex '.*web[0-9]+$') -gt 0 ]]; then
    call "Crons are beeing backed up to: ${cron_ba}/."
    if [[ ! -d ${cron_ba} ]]; then
      mkdir ${cron_ba}
    fi
  else
    call "0 Crons found. Nothing to move around here..."
    exit 0
  fi

  call "Checking cron.d ispc_xxx files."
  for i in $(find /etc/cron.d/ -iname "ispc*"); do
    if [[ -f ${i} ]]; then
      call "Moving ${i} to ${cron_ba}/"
      mv ${i} ${cron_ba}/
    fi
  done
  
  call "Checking /var/spool/cron/crontabs/ webxxx files"
  for i in $(find /var/spool/cron/crontabs/ -regex '.*web[0-9]+$'); do
    if [[ -f ${i} ]]; then
      call "Moving ${i} to ${cron_ba}/"
      mv ${i} ${cron_ba}/
    fi
  done
  exit 0
fi

vhost_ba="/etc/nginx/${tstamp}-sites-semi-available"
call "vHosts are beeing backed up to: ${vhost_ba}/."
call "In case you need to restore things, just run 'mv ${vhost_ba}/*.vhost /etc/nginx/sites-available/'"
for i in  $(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select domain from dbispconfig.web_domain WHERE vhost_type = 'name' AND server_id = ${srv_id};"); do
  domains+=("$i")
done

if [[ ${mode} == "ISPCopyProxy" ]]; then
  ispc_ini_file="/root/ispcopy/ispcopy_input.ini"
  if [[ ! -f ${ispc_ini_file} ]]; then
    call "${mode} requires ${ispc_ini_file} to be available. Make sure the file is in place."
    call "Output of ls: \n-----\n$(ls -lah "/root/ispcopy/")\n-----"
    exit 1
  else
    call "Using ISPCopy .ini file: ${ispc_ini_file} for Proxy Mapping."
  fi
  for line in $(awk '/replace_ip_/' ${ispc_ini_file}); do
    oIP=$(echo "${line}" | awk -F "=" '{print$1}' | sed 's/replace_ip_//g' | sed 's/_/./g');
    nIP=$(echo "${line}" | awk -F "=" '{print$2}');
    ip_map[${oIP}]+="${nIP}"
  done
  target_main_ip=$(awk -F "=" '/target_server_ip/{print$2}' ${ispc_ini_file})
  if [[ -z ${target_main_ip} ]]; then
    call "Cant find new servers default IP. Need to exit here"
    exit 1
  else
    call "New Servers Main IP: ${target_main_ip}"
  fi
fi

## Generating the vHost ..
for d in ${domains[@]}; do
  call "Getting details for domain: ${d}"
  ip=$(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select ip_address from dbispconfig.web_domain where domain = '${d}' AND server_id = ${srv_id};")
  d_id=$(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select domain_id from dbispconfig.web_domain where domain = '${d}' AND server_id = ${srv_id};")
  
  if [[ ${mode} == "ISPCopyProxy" ]]; then
   for x in "${!ip_map[@]}"; do
     if [[ ${x} == ${ip} ]]; then
       old_ip=${ip};
       proxy_ip=${ip_map[$ip]}
     fi;
   done
   if [[ ! -z ${old_ip} && ! -z ${proxy_ip} ]]; then
     call "✔ ${old_ip} will be proxied to: ${proxy_ip}"
   else
     call "✔ No Mapping found for ${ip}. Using ${target_main_ip} as fallback."
     proxy_ip=${target_main_ip}
   fi
  fi

  p_domain=${d}
  p_d_id=${d_id}
  ssl_mode=$(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select \`ssl\` from dbispconfig.web_domain where domain_id = '${p_d_id}' AND server_id = ${srv_id};")
  server_names+=("${p_domain}")

  www=$(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select is_subdomainwww from dbispconfig.web_domain where domain = '${d}' AND server_id = ${srv_id};")

  [[ ${www} == 1 ]] && server_names+=("www.${d}")

  # AliasDomains and SubDomains
  for i in $(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select domain from dbispconfig.web_domain where parent_domain_id = '${p_d_id}' AND server_id = ${srv_id};"); do
    if [[ ! -z ${i} ]]; then
      server_names+=("${i}")
      # add WWW to alias if exists
      if [[ $(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select is_subdomainwww from dbispconfig.web_domain where subdomain = 'www' AND domain = '${i}' AND server_id = ${srv_id};") == 1 ]]; then
        server_names+=("www.${i}")
      fi
    fi
  done
  
  
  root="/var/www/${d}/web"
  err="/var/log/ispconfig/httpd/${d}/error.log"
  acc="/var/log/ispconfig/httpd/${d}/access.log"
  web_path=$(mysql --defaults-extra-file=/etc/mysql/debian.cnf -N -e "select document_root from dbispconfig.web_domain where domain = '${d}';")
  ssl_scheme="http"
  ssl=""
  
  if [[ ! -z ${web_path} && ${ssl_mode} == "y" ]]; then
    [[ -f "${web_path}/ssl/${d}.crt" ]] && ssl_crt="ssl_certificate ${web_path}/ssl/${d}.crt;" || ssl_crt=""
    [[ -f "${web_path}/ssl/${d}.key" ]] && ssl_key="ssl_certificate_key ${web_path}/ssl/${d}.key;" || ssl_key=""
    if [[ ! -z ${ssl_key} && ! -z ${ssl_crt} ]]; then
      ssl_scheme="https"
      ssl="
  listen ${ip}:443 ssl http2;
  ssl_protocols TLSv1.2 TLSv1.3;
  ${ssl_crt}
  ${ssl_key}
"
    fi
  fi
  
  if [[ -z ${ssl_scheme} ]]; then
    call "cant tell wether http or https. Exiting here!"
    exit 1
  fi

  if [[ ${maint} ]]; then
    [[ ! -d "${vhost_ba}" ]] && mkdir "${vhost_ba}"
    [[ -f "/etc/nginx/sites-available/${d}.vhost" ]] && mv "/etc/nginx/sites-available/${d}.vhost" "${vhost_ba}/${d}.vhost"
    call "✔ Fixing maintenance mode vHost for domain: ${d}!"
    
    echo -e "server {
listen ${ip}:80;
${ssl}
server_name ${server_names[*]};
root   ${root};
error_log ${err};
access_log ${acc} combined;
index index.html index.htm index.php index.cgi index.pl index.xhtml;

set \$maintenance_page th-maintenance.html;
set \$custom_root /var/www/html;
set \$doc_name TH-Doc;

if (-f \$document_root/custom-maintenance.html) {
   set \$maintenance_page custom-maintenance.html;
   set \$custom_root \$document_root;
   set \$doc_name Custom-Doc;
}

error_page 503 @th-maintenance;

location @th-maintenance {
  root \$custom_root;
  rewrite ^(.*)\$ /\$maintenance_page break;
  add_header Retry-After 120 always;
  add_header X-Maintenance \$doc_name always;
}
return 503;
}"  > "/etc/nginx/sites-available/${d}.vhost"
  fi

  if [[ ! -z ${proxy_ip} ]]; then
    r_ip=$(echo "${proxy_ip}" | sed -E -e 's/[[:blank:]]+//g'| grep -Eo "${ipmatch}")
    [[ ${proxy_ip} == ${r_ip} ]] && reverse_ip=${r_ip} || exit 1
    call "✔ Fixing ReverseProxy vHost for domain: ${d}!"
    [[ ! -d "${vhost_ba}" ]] && mkdir "${vhost_ba}"
    [[ -f "/etc/nginx/sites-available/${d}.vhost" ]] && mv "/etc/nginx/sites-available/${d}.vhost" "${vhost_ba}/${d}.vhost"
      
      
    echo -e "server {
listen ${ip}:80;
${ssl}
server_name ${server_names[*]};
root   ${root};
error_log ${err};
access_log ${acc} combined;

  location / {
    proxy_pass ${ssl_scheme}://${reverse_ip}/;
    rewrite ^//(.*) /\$1;
    proxy_set_header X-Forwarded-Proto \$scheme;
    proxy_set_header Host \$host;
    proxy_set_header X-Real-IP \$remote_addr;
    proxy_set_header X-Forwarded-For \$proxy_add_x_forwarded_for;
    proxy_pass_header Authorization;
    client_max_body_size 0;
    client_body_buffer_size 1m;
    proxy_intercept_errors on;
    proxy_buffering on;
    proxy_buffer_size 128k;
    proxy_buffers 256 16k;
    proxy_busy_buffers_size 256k;
    proxy_temp_file_write_size 256k;
    proxy_max_temp_file_size 0;
    proxy_read_timeout 300;
  }
}"  > "/etc/nginx/sites-available/${d}.vhost"
    fi
      ssl=""
  unset server_names
  if [[ ${mode} == "ISPCopyProxy" ]]; then
    unset proxy_ip
    unset old_ip
  fi
  unset ssl_mode
done


call "nginx -t Result:"
nginx -t

service nginx restart
